using Microsoft.UI.Xaml;
using Microsoft.UI.Xaml.Controls;
using Microsoft.UI.Xaml.Controls.Primitives;
using Microsoft.UI.Xaml.Data;
using Microsoft.UI.Xaml.Input;
using Microsoft.UI.Xaml.Media;
using Microsoft.UI.Xaml.Navigation;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Runtime.InteropServices.WindowsRuntime;
using Windows.Foundation;
using Windows.Foundation.Collections;
using WinRT;
using LiftoffSDK.Public;
using System.Collections.Concurrent;
using Microsoft.VisualBasic;
using LiftoffSDK;


// To learn more about WinUI, the WinUI project structure,
// and more about our project templates, see: http://aka.ms/winui-project-info.

namespace LiftoffWinUI3DemoApp
{
    /// <summary>
    /// An empty window that can be used on its own or navigated to within a Frame.
    /// </summary>
    public sealed partial class MainWindow : Window
    {
        private LiftoffAds? m_sdk = null;

        // This dictionary let's us know which placements are loaded ("ready to play"), and which aren't
        private ConcurrentDictionary<String, bool> m_placementLoadState = new ConcurrentDictionary<string, bool>();

        public MainWindow()
        {
            this.InitializeComponent();

            btnStartVungle.Click += BtnStartVungle_Click;
            btnLoad.Click += BtnLoad_Click;
            btnPlay.Click += BtnPlay_Click;
            listViewPlacements.SelectionChanged += ListViewPlacements_SelectionChanged;
        }

        private async void BtnStartVungle_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                // You can only initialize the SDK once.
                // If your initialization fails (e.g. because appID or endpoint was incorrect),
                // then you will need to restart the app and try again.
                this.btnStartVungle.IsEnabled = false;

                String appID = this.tbAppId.Text.Trim();
                String endpoint = this.tbApiEndpoint.Text.Trim();

                LiftoffSdkConfig sdkConfig = new LiftoffSdkConfig();
                // The publisher may wish to use a custom endpoint for testing
                if (!String.IsNullOrWhiteSpace(endpoint))
                {
                    sdkConfig.ApiEndpoint = new Uri(endpoint);
                }

                // Use this to get debugging information
                LiftoffAds.AddDiagnosticListener(LiftoffAds_OnDiagnostic);

                // We want to get a callback so that we know if we were able to initialize the SDK
                // or not.
                LiftoffAds.OnInitializationSuccess += LiftoffAds_OnInitializationSuccess;
                LiftoffAds.OnInitializationFailure += LiftoffAds_OnInitializationFailure;
                // We want to get a callback so that we know if the ad was loaded or not
                LiftoffAds.OnAdPlayableChanged += LiftoffAds_OnAdPlayableChanged;
                LiftoffAds.OnAdStart += LiftoffAds_OnAdStart;
                LiftoffAds.OnAdEnd += LiftoffAds_OnAdEnd;
                // Now, let's initialize the SDK
                m_sdk = await LiftoffAds.InitializeAsync(appID, sdkConfig, this);
            }
            catch (Exception ex)
            {
                ShowUserMessage(ex.ToString());
            }
        }

        private void LiftoffAds_OnAdStart(object sender, AdStartEventArgs e)
        {
            System.Diagnostics.Debug.WriteLine("Ad has started playing...");
            System.Diagnostics.Debug.WriteLine(e.ToString());
        }

        private void LiftoffAds_OnAdEnd(object sender, AdEndEventArgs e)
        {
            System.Diagnostics.Debug.WriteLine("Ad has finished playing...");
            System.Diagnostics.Debug.WriteLine(e.ToString());
        }

        private void BtnLoad_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                var placement = this.listViewPlacements.SelectedItem as String;

                if (placement is null)
                {
                    ShowUserMessage("Please select a placement from the list before clicking on 'Load'");
                    return;
                }
                m_sdk?.LoadAd(placement);
            }
            catch (Exception ex)
            {
                ShowUserMessage(ex.ToString());
            }
        }

        private async void BtnPlay_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                var placement = this.listViewPlacements.SelectedItem as String;

                if (placement is null)
                {
                    ShowUserMessage("Please select a placement from the list before clicking on 'Play'");
                    return;
                }
                AdConfig adConfig = new AdConfig();
                // if (Lift ViewModels.MainDataContext.s_)
                // LiftoffWinTestAppSharedCode.
                var playInfo = await m_sdk?.PlayAdAsync(adConfig, placement);
                if (playInfo != null)
                {
                    if (playInfo.Success == true)
                    {
                        System.Diagnostics.Debug.WriteLine(playInfo.ToString());
                    }
                    else
                    {
                        ShowUserMessage(playInfo.ToString());
                    }
                }
            }
            catch (Exception ex)
            {
                ShowUserMessage(ex.ToString());
            }
        }

        private void LiftoffAds_OnInitializationSuccess(object? sender, InitializationSuccessEventArgs e)
        {
            try
            {
                this.listViewPlacements.Items.Clear();
                // Display the names of the various placements
                foreach (var placement in e.Placements)
                {
                    this.listViewPlacements.Items.Add(placement.Name);
                }
                // Now that we are initialized, we can allow the user to load ads.
                this.btnLoad.IsEnabled = true;
            }
            catch (Exception ex)
            {
                ShowUserMessage(ex.ToString());
            }
        }

        private void LiftoffAds_OnInitializationFailure(object? sender, InitializationFailureEventArgs e)
        {
            ShowUserMessage("Initialization failed.  Please check your appID and endpoint.");
        }

        private void LiftoffAds_OnAdPlayableChanged(object? sender, AdPlayableEventArgs e)
        {
            try
            {
                // If the previously selected ad is now playable, we would like to enable the "Play" button
                m_placementLoadState[e.Placement] = e.AdPlayable;
                String? selectedItem = this.listViewPlacements.SelectedItem as String;
                if (selectedItem == null)
                {
                    return;
                }
                this.btnPlay.IsEnabled = m_placementLoadState[selectedItem];
            }
            catch (Exception ex)
            {
                ShowUserMessage(ex.ToString());
            }
        }


        private void ListViewPlacements_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            try
            {
                var placement = e.AddedItems[0] as String;
                if (placement is null)
                {
                    return;
                }

                bool enabled;
                m_placementLoadState.TryGetValue(placement, out enabled);
                this.btnPlay.IsEnabled = enabled;
            }
            catch (Exception ex)
            {
                ShowUserMessage(ex.ToString());
            }
        }

        private async void ShowUserMessage(String message)
        {
            try
            {
                ContentDialog messageBox = new ContentDialog
                {
                    Title = "Debug Message",
                    Content = message,
                    PrimaryButtonText = "OK"
                };
                messageBox.XamlRoot = btnLoad.XamlRoot;
                await messageBox.ShowAsync();
            }
            catch (Exception ex)
            {
                System.Diagnostics.Debug.WriteLine(ex.ToString());
            }
        }

        private void LiftoffAds_OnDiagnostic(object sender, DiagnosticLogEvent e)
        {
            System.Diagnostics.Debug.WriteLine(e.ToString());
        }

    }
}
